#include <iostream>
#include <iomanip>
#include <Freeimage.h>

int main(int argc, char**argv)
{
    FIBITMAP* dib;
	FREE_IMAGE_FORMAT fif = FIF_UNKNOWN;
	char *lpszPathName;
	RGBQUAD *paleta;
	RGBQUAD fondo;

	if(argc < 2) {
        printf("Usar: FreeImage002 <fichero>\n");
        return 0;
	}
	lpszPathName = argv[1];

	fif = FreeImage_GetFileType(lpszPathName, 0);
	if(fif == FIF_UNKNOWN) {
		fif = FreeImage_GetFIFFromFilename(lpszPathName);
	}
	if((fif != FIF_UNKNOWN) && FreeImage_FIFSupportsReading(fif)) {
		FIBITMAP *dib = FreeImage_Load(fif, lpszPathName, 0);
		std::cout << "Fichero: " << lpszPathName << std::endl;

        std::cout << "Tipo: " << FreeImage_GetImageType(dib) << std::endl;
        std::cout << "Colores: " << FreeImage_GetColorsUsed(dib) << std::endl;
        std::cout << "Bits por pixel: " << FreeImage_GetBPP(dib) << std::endl;
        std::cout << "Anchura: " << FreeImage_GetWidth(dib) << std::endl;
        std::cout << "Altura: " << FreeImage_GetHeight(dib) << std::endl;
        std::cout << "Anchura de linea:" << FreeImage_GetLine(dib) << std::endl;
        std::cout << "Pitch: " << FreeImage_GetPitch(dib) << std::endl;
        std::cout << "DIBSize: " << FreeImage_GetDIBSize(dib) << std::endl;
        std::cout << "MemorySize: " << FreeImage_GetMemorySize(dib) << std::endl;
        paleta = FreeImage_GetPalette(dib);
        if(paleta) {
            for(int i = 0; i < FreeImage_GetColorsUsed(dib); i++) {
                std::cout << "Color "<< std::dec << std::setw(3) << std::setfill('0') << i << ": " <<
                std::setw(2) << std::setfill('0') <<  std::hex << static_cast<int>(paleta[i].rgbRed) <<
                std::setw(2) << std::setfill('0') <<  std::hex << static_cast<int>(paleta[i].rgbGreen) <<
                std::setw(2) << std::setfill('0') <<  std::hex << static_cast<int>(paleta[i].rgbBlue) << std::endl;
            }
        }
        else { std::cout << "No hay paleta" << std::endl; }
        std::cout << "Puntos por Metro X: " << std::dec << FreeImage_GetDotsPerMeterX(dib) << std::endl;
        std::cout << "Puntos por Metro Y: " << FreeImage_GetDotsPerMeterY(dib) << std::endl;
        switch(FreeImage_GetColorType(dib)) {
        case FIC_MINISBLACK:
            std::cout << "Monocromo, primera entrada en paleta es color negro" << std::endl;
            break;
        case FIC_MINISWHITE:
            std::cout << "Monocromo, primera entrada en paleta es color blanco" << std::endl;
            break;
        case FIC_PALETTE:
            std::cout << "Mapa de bits con paleta" << std::endl;
            break;
        case FIC_RGB:
            std::cout << "Mapa de bits RGB" << std::endl;
            break;
        case FIC_RGBALPHA:
            std::cout << "Mapa de bits RGB con canal alfa" << std::endl;
            break;
        case FIC_CMYK:
            std::cout << "Mapa de bits CNYK" << std::endl;
            break;
        }
        unsigned mascara = FreeImage_GetRedMask(dib);
        if(mascara) {
            std::cout << "Mascara de  rojo " << std::setw(8) << std::setfill('0') <<  std::hex << mascara << std::endl;
        } else { std::cout << "Mapa con paleta, no hay mascara" << std::endl; }
        mascara = FreeImage_GetGreenMask(dib);
        if(mascara) {
            std::cout << "Mascara de verde " << std::setw(8) << std::setfill('0') <<  std::hex << mascara << std::endl;
        }
        mascara = FreeImage_GetBlueMask(dib);
        if(mascara) {
            std::cout << "Mascara de  azul " << std::setw(8) << std::setfill('0') <<  std::hex << mascara << std::endl;
        }
        std::cout << "Colores transparentes: "<< std::dec << FreeImage_GetTransparencyCount(dib) << std::endl;
        BYTE *tabla = FreeImage_GetTransparencyTable(dib);
        if(tabla != NULL && FreeImage_GetTransparencyCount(dib) > 0) {
            for(int i=0; i<FreeImage_GetTransparencyCount(dib); i++)
                std::cout << "Tabla(" << i << ") " << static_cast<int>(tabla[i]) << std::endl;
        }
        if(FreeImage_IsTransparent(dib)) std::cout << "Tabla de transparencia habilitada" << std::endl;
        else std::cout << "Tabla de transparencia no habilitada" << std::endl;
        std::cout << "Indice de transparencia " << FreeImage_GetTransparentIndex(dib) << std::endl;
        if(FreeImage_HasBackgroundColor(dib)) {
            FreeImage_GetBackgroundColor(dib, &fondo);
            std::cout << "Existe color de fondo: " <<
                std::setw(2) << std::setfill('0') <<  std::hex << static_cast<int>(fondo.rgbRed) <<
                std::setw(2) << std::setfill('0') <<  std::hex << static_cast<int>(fondo.rgbGreen) <<
                std::setw(2) << std::setfill('0') <<  std::hex << static_cast<int>(fondo.rgbBlue) << std::endl;
        } else std::cout << "No existe color de fondo" << std::endl;

        if(FreeImage_HasPixels(dib)) std::cout << "El dib contiene informacion de pixels." << std::endl;
        else std::cout << "El dib no contiene informacion de pixels." << std::endl;
		FreeImage_Unload(dib);
	}

    return 0;
}
